/*###############################################################################
## ETiles project
##
## Copyright (C) 2010 Carlos Villegas Núñez  carlos.villegasnunez@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################*/

#include "mapa.h"

#include <QtGui>
#include <iostream>

const double zoomposibles[7] = {0.2,0.5,0.75,1,1.25,1.5,2};

Mapa::Mapa(VentanaPrincipal *parent ,int tanchura,int taltura, int tnumanchura, int tnumaltura) : ventana(parent)
{
     setAcceptDrops(true);
     setScaledContents(true);
     setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
     setMouseTracking(1);
     setFixedSize(tanchura*tnumanchura,taltura*tnumaltura);
     this->tanchura=tanchuraZ=tanchura;
     this->taltura=talturaZ=taltura;
     this->tnumaltura=tnumaltura;
     this->tnumanchura=tnumanchura;

     matrizTiles = new int*[tnumanchura];
     for(int i=0;i<tnumanchura;i++)
     {
         matrizTiles[i] = new int[tnumaltura];
     }
     for(int i=0;i<tnumanchura;i++)
         for(int j=0;j<tnumaltura;j++)
             matrizTiles[i][j]=-1;
     factorzoom=3;

     crejilla = QColor(Qt::white);
     cfondo = QColor(Qt::black);
     csel = QColor(240,240,240,125);

     /*prueba.show();
     prueba.setWindowTitle("X");
     prueba.move(QPoint(500,50));

     prueba1.show();
     prueba1.setWindowTitle("Y");
     prueba1.move(QPoint(600,50));*/

     contador=0;contador1=0; modificado=1;  pintarc=1;  pintarcompleto=0;
     seleccion=false;borrar=false;relleno=false;rellenoConjunto=false;
     rejillaactiva=true; rejillapintada=false;pegado=false;mov=false;

     rectselant=QRect(0,0,tanchura,taltura);
     rectsel=QRect(0,0,tanchuraZ,talturaZ);
}

void Mapa::SeleccionActivar(bool i)
{
    QRect rect = rectsel;
    rectsel = QRect();
    update(rect);

    setMouseTracking(!i);
    seleccion=i;
}

void Mapa::RellenoActivar(bool i)
{
    QRect rect = rectsel;
    rectsel = QRect();
    update(rect);

    setMouseTracking(!i);
    relleno=i;
}

void Mapa::RellenoConjuntoActivar(bool i)
{
    QRect rect = rectsel;
    rectsel = QRect();
    update(rect);

    setMouseTracking(!i);
    rellenoConjunto=i;
}

void Mapa::PegarActivar(bool i)
{
    if(i==true)
    {
        if(rectcopiar.isValid())
        {
            setMouseTracking(1);
            pegado=true;
         }
    }
    else
    {
        if(seleccion || borrar || relleno)
           setMouseTracking(0);
        else
           setMouseTracking(1);
        pegado=false;
    }
    update(rectcopiar);
}

void Mapa::BorrarActivar(bool i)
{
    QRect rect = rectsel;
    rectsel = QRect();
    update(rect);

    setMouseTracking(!i);
    borrar=i;
}

void Mapa::ActivarRejilla(bool i)
{
    if(i)
    {
        rejillaactiva=true;
        rejillapintada=false;
    }
    else
    {
        rejillaactiva=false;
        pintarcompleto=1;
    }
    update();
}

void Mapa::AsignarListaTile(Tile *tiles)
{
    listatiles=tiles;
}

void Mapa::Deshacer()
{
    QPoint numtiles;
    if(!numdeshacer.isEmpty())
    {
          numtiles = numdeshacer.back();
          numdeshacer.pop_back();
          //numrehacer.append(numtiles);
          InsertarNumRehacer(numtiles.x(),numtiles.y());
          if(numtiles.x()==0) // 0 cuando se inserto algun tile
          {
              for(int i=0;i<numtiles.y();i++)
              {
                  //Tomamos el ultimo elemento y luego lo borramos de la lista deshacer.
                  int tile = tilesdeshacer.back();
                  QPoint pos = posdeshacer.back();
                  tilesdeshacer.pop_back();
                  posdeshacer.pop_back();

                  QRect rect(pos.x()*tanchuraZ,pos.y()*talturaZ,tanchuraZ,talturaZ);
                  //Buscamos el tile en la lista tiles para borrarlo.

                  matrizTiles[pos.x()][pos.y()] = -1;

                  minimapa->Actualizar(QPoint(pos.x(),pos.y()),false);
                  InsertarRehacer(tile,pos);

                  update(rect);
              }
          }
          else if(numtiles.x()==1) //Si se borro algun tile
          {
              for(int i=0;i<numtiles.y();i++)
              {
                    //Tomamos el ultimo elemento y luego lo borramos de la lista deshacer.
                    int tile = tilesdeshacer.back();
                    QPoint pos = posdeshacer.back();
                    tilesdeshacer.pop_back();
                    posdeshacer.pop_back();

                    QRect rect(pos.x()*tanchuraZ,pos.y()*talturaZ,tanchuraZ,talturaZ);

                    matrizTiles[pos.x()][pos.y()] = tile;

                    minimapa->Actualizar(QPoint(pos.x(),pos.y()),listatiles->ImagenTile(tile),true);

                    InsertarRehacer(tile,pos);
                    update(rect);
              }
          }
          else if(numtiles.x() == 2) //Si se movio algun tile.
          {
                QPoint pos = posmovact.back();
                int tile = tilemov.back();
                posmovact.pop_back();
                tilemov.pop_back();

                posmovactr.push_back(pos);
                tilemovr.push_back(tile);

                int x=pos.x()*tanchuraZ,y=pos.y()*talturaZ;
                QRect rect(x,y,tanchuraZ,talturaZ);

                QPoint posn = posmovant.back();
                posmovant.pop_back();

                posmovantr.push_back(posn);

                int xnuevo=posn.x()*tanchuraZ,ynuevo=posn.y()*talturaZ;
                QRect rectn(xnuevo,ynuevo,tanchuraZ,talturaZ);

                matrizTiles[posn.x()][posn.y()] = tile;
                minimapa->Actualizar(QPoint(posn.x(),posn.y()),listatiles->ImagenTile(matrizTiles[pos.x()][pos.y()]),true);
                
                matrizTiles[pos.x()][pos.y()] = -1;
                minimapa->Actualizar(QPoint(pos.x(),pos.y()),false);
                
                update(rect);
                update(rectn);

          }
    }
}

void Mapa::Rehacer()
{
    QPoint numtiles;
    if(numrehacer.size() > 0)
    {
          numtiles = numrehacer.back();
          numrehacer.pop_back();
          InsertarNumDeshacer(numtiles.x(),numtiles.y());
          //numdeshacer.append(numtiles);
          if(numtiles.x()==0) // 0 cuando se inserto algun tile, volvemos a insertarlo.
          {
              for(int i=0;i<numtiles.y();i++)
              {
                    //Tomamos el ultimo elemento y luego lo borramos de la lista deshacer.
                    int tile = tilesrehacer.back();
                    QPoint pos = posrehacer.back();
                    tilesrehacer.pop_back();
                    posrehacer.pop_back();

                    QRect rect(pos.x()*tanchuraZ,pos.y()*talturaZ,tanchuraZ,talturaZ);

                    matrizTiles[pos.x()][pos.y()] = tile;
                    //matrizRect[pos.x()][pos.y()] = rect;
                    minimapa->Actualizar(QPoint(pos.x(),pos.y()),listatiles->ImagenTile(tile),true);

                    InsertarDeshacer(tile,pos);
                    update(rect);
              }
          }
          else if(numtiles.x()==1) //Si se borro algun tile, volvemos a borrarlo
          {
              for(int i=0;i<numtiles.y();i++)
              {
                  //Tomamos el ultimo elemento y luego lo borramos de la lista deshacer.
                  int tile = tilesrehacer.back();
                  QPoint pos = posrehacer.back();
                  tilesrehacer.pop_back();
                  posrehacer.pop_back();

                  QRect rect(pos.x()*tanchuraZ,pos.y()*talturaZ,tanchuraZ,talturaZ);
                  //Buscamos el tile en la lista tiles para borrarlo.

                  matrizTiles[pos.x()][pos.y()] = -1;
                  //matrizRect[pos.x()][pos.y()] = QRect();
                  minimapa->Actualizar(QPoint(pos.x(),pos.y()),false);

                  InsertarDeshacer(tile,pos);

                  update(rect);
              }
          }
          else if(numtiles.x() == 2) //Si se movio algun tile.
          {
              QPoint pos = posmovantr.back();
              int tile = tilemovr.back();
              posmovantr.pop_back();
              tilemovr.pop_back();

              posmovant.push_back(pos);
              tilemov.push_back(tile);

              int x=pos.x()*tanchuraZ,y=pos.y()*talturaZ;
              QRect rect(x,y,tanchuraZ,talturaZ);

              QPoint posn = posmovactr.back();
              posmovactr.pop_back();

              posmovact.push_back(posn);

              int xnuevo=posn.x()*tanchuraZ,ynuevo=posn.y()*talturaZ;
              QRect rectn(xnuevo,ynuevo,tanchuraZ,talturaZ);

              matrizTiles[posn.x()][posn.y()] = tile;
              minimapa->Actualizar(QPoint(posn.x(),posn.y()),listatiles->ImagenTile(matrizTiles[pos.x()][pos.y()]),true);

              matrizTiles[pos.x()][pos.y()] = -1;
              minimapa->Actualizar(QPoint(pos.x(),pos.y()),false);

              update(rect);
              update(rectn);
          }
    }
}

void Mapa::ActualizarDeshacer(int tile)
{
    int indice=0;
    int indicemov=0;
    QList<int> ldes,lima,lmov;
    for(int i=0; i<numdeshacer.size();i++)
    {
        if(numdeshacer.at(i).x()==2)
        {
            if(tilemov.at(indicemov)==tile)
            {
                lmov.push_back(indicemov);
                ldes.push_back(i);
            }
            else if(tilemov.at(indicemov)>tile)
            {
                tilemov[indicemov] -= 1;
            }
            indicemov++;
        }
        else
        {
            int nummod=0;
            for(int j=0; j<numdeshacer.at(i).y(); j++) //Para cada tile de la accion producida
            {
                if(tilesdeshacer.at(indice) == tile)
                {
                    nummod++;
                    lima.push_back(indice);
                }
                else if(tilesdeshacer.at(indice) > tile)
                {
                    tilesdeshacer[indice] -= 1;
                }
                indice++;
            }
            if(numdeshacer.at(i).y() == nummod)
            {
                ldes.push_back(i);
            }
            else //Si no se han eliminado todos, restamos los eliminados
            {
                numdeshacer[i].setY(numdeshacer[i].y()-nummod);
            }
        }
     }
    for(int i=ldes.size();i>0;i--)
        numdeshacer.removeAt(ldes.at(i-1));
    for(int i=lima.size();i>0;i--)
    {
        tilesdeshacer.removeAt(lima.at(i-1));
        posdeshacer.removeAt(lima.at(i-1));
    }
    for(int i=lmov.size();i>0;i--)
    {
        posmovant.removeAt(lmov.at(i-1));
        posmovact.removeAt(lmov.at(i-1));
        tilemov.removeAt(lmov.at(i-1));
    }
}

void Mapa::ActualizarRehacer(int tile)
{
    int indice=0;
    int indicemov=0;
    QList<int> ldes,lima,lmov;
    for(int i=0; i<numrehacer.size();i++)
    {
        if(numrehacer.at(i).x()==2)
        {
            if(tilemovr.at(indicemov)==tile)
            {
                lmov.push_back(indicemov);
                ldes.push_back(i);
            }
            else if(tilemovr.at(indicemov)>tile)
            {
                tilemovr[indicemov] -= 1;
            }
            indicemov++;
        }
        else
        {
            int nummod=0;
            for(int j=0; j<numrehacer.at(i).y(); j++)
            {
                if(tilesrehacer.at(indice) == tile)
                {
                    nummod++;
                    lima.push_back(indice);
                }
                else if(tilesrehacer.at(indice) > tile)
                {
                    tilesrehacer[indice] -= 1;
                }
                indice++;
            }
            if(numrehacer.at(i).y() == nummod)
            {
                ldes.push_back(i);
            }
            else //Si no se han eliminado todos, restamos los eliminados
            {
                numrehacer[i].setY(numrehacer[i].y()-nummod);
            }
        }
     }
    for(int i=ldes.size();i>0;i--)
        numrehacer.removeAt(ldes.at(i-1));
    for(int i=lima.size();i>0;i--)
    {
        tilesrehacer.removeAt(lima.at(i-1));
        posrehacer.removeAt(lima.at(i-1));
    }
    for(int i=lmov.size();i>0;i--)
    {
        posmovantr.removeAt(lmov.at(i-1));
        posmovactr.removeAt(lmov.at(i-1));
        tilemovr.removeAt(lmov.at(i-1));
    }
}

void Mapa::InsertarRehacer(int tile, QPoint pos)
{
    tilesrehacer.append(tile);
    posrehacer.append(pos);
}

void Mapa::InsertarDeshacer(int tile, QPoint pos)
{
    tilesdeshacer.append(tile);
    posdeshacer.append(pos);
}

void Mapa::InsertarNumRehacer(int tipo,int num)
{
    if(numrehacer.size() == 100)
    {
        if(numrehacer.front().x() == 2) //Se trata de un movimiento
        {
            posmovactr.pop_front();
            tilemovr.pop_front();
        }
        else
        {
            for(int i=0; i < numrehacer.front().y(); i++)
            {
                tilesrehacer.pop_front();
                posrehacer.pop_front();
            }
        }
        numrehacer.pop_front();
        numrehacer.append(QPoint(tipo,num));
    }
    else
        numrehacer.append(QPoint(tipo,num));
}

void Mapa::InsertarNumDeshacer(int tipo, int num)
{
    if(numdeshacer.size() == 100)
    {
        if(numdeshacer.front().x() == 2) //Se trata de un movimiento
        {
            posmovact.pop_front();
            tilemov.pop_front();
        }
        else
        {
            for(int i=0; i < numdeshacer.front().y(); i++)
            {
                tilesdeshacer.pop_front();
                posdeshacer.pop_front();
            }
        }
        numdeshacer.pop_front();
        numdeshacer.append(QPoint(tipo,num));
    }
    else{
            numdeshacer.append(QPoint(tipo,num));
    }
}

void Mapa::Borrar(QRect rect)
{
    int fila= (int)(rect.x()/tanchuraZ);
    int columna= (int)(rect.y()/talturaZ);
    if(!posLibre(fila,columna))
    {
        InsertarNumDeshacer(1,1);
        InsertarDeshacer(matrizTiles[fila][columna],QPoint(fila,columna));

        minimapa->Actualizar(QPoint(fila,columna),false);

        matrizTiles[fila][columna]=-1;
    }

    update(rect);
}

void Mapa::Rellenar(QRect rect)
{
    int i = listatiles->vistalistatiles->TileSeleccionado();
    //Si tenemos un tile seleccionado
    if(i!=-1)
    {
        QPixmap pixmap = listatiles->ImagenTile(i); //Guardamos el tile seleccionado en la ventana tiles.
        int fila= (int)(rect.x()/tanchuraZ);
        int columna= (int)(rect.y()/talturaZ);
        if(posLibre(fila,columna))
        {
            matrizTiles[fila][columna]=i;

            minimapa->Actualizar(QPoint(fila,columna),pixmap,true);

            InsertarNumDeshacer(0,1);
            InsertarDeshacer(i,QPoint(rect.x()/tanchuraZ,rect.y()/talturaZ));
        }
    }
    update(rect);
}

int Mapa::RellenarRecursiva(QPoint pos)
{
    int i = listatiles->vistalistatiles->TileSeleccionado();
    //Si tenemos un tile seleccionado
    if(i!=-1)
    {
        if(posCorrecta(QPoint(pos.x()*tanchuraZ,pos.y()*talturaZ)))
        {
            if(posLibre(pos.x(),pos.y()))
            {
                QPixmap pixmap = listatiles->ImagenTile(i);

                matrizTiles[pos.x()][pos.y()]=i;

                int num1= RellenarRecursiva(QPoint(pos.x()+1,pos.y()));
                int num2= RellenarRecursiva(QPoint(pos.x()-1,pos.y()));
                int num3= RellenarRecursiva(QPoint(pos.x(),pos.y()+1));
                int num4= RellenarRecursiva(QPoint(pos.x(),pos.y()-1));

                update(RectanguloPosicion(pos.x(),pos.y()));
                minimapa->Actualizar(QPoint(pos.x(),pos.y()),pixmap,true);

                InsertarDeshacer(i,QPoint(pos.x(),pos.y()));

                return num1+num2+num3+num4+1;
            }
        }
    }
    return 0;
}

void Mapa::RellenarConjunto(QPoint pos)
{
    int i = listatiles->vistalistatiles->TileSeleccionado();
    //Si tenemos un tile seleccionado
    if(i!=-1)
    {
        if(posLibre(pos.x(),pos.y()))
        {
            QPixmap pixmap = listatiles->ImagenTile(i);

            matrizTiles[pos.x()][pos.y()]=i;

            int num1= RellenarRecursiva(QPoint(pos.x()+1,pos.y()));
            int num2= RellenarRecursiva(QPoint(pos.x()-1,pos.y()));
            int num3= RellenarRecursiva(QPoint(pos.x(),pos.y()+1));
            int num4= RellenarRecursiva(QPoint(pos.x(),pos.y()-1));

            update(RectanguloPosicion(pos.x(),pos.y()));
            minimapa->Actualizar(QPoint(pos.x(),pos.y()),pixmap,true);

            InsertarDeshacer(i,QPoint(pos.x(),pos.y()));
            InsertarNumDeshacer(0,num1+num2+num3+num4+1);
        }
    }
}


void Mapa::Copiar()
{
    int x=0,y=0; //Tile (0,0) de la region copiada, necesaria para copiar solo los tiles pintados de la region porque siempre va a ser un rectangulo
    if(rectsel.isValid())
    {
        rectcopiar=rectsel;
        tilescopiar = QList<int>();
        poscopiar=QList<QPoint>();
        //si la region a copiar es mayor que un solo tile, debemos ir copiando una a uno
        if(rectsel.width() > tanchuraZ || rectsel.height() > talturaZ)
        {
            QPoint pos = rectsel.topLeft();
            while(pos.x()<=rectsel.right())
            {
                while(pos.y()<=rectsel.bottom())
                {
                    QRect tile = TileSeleccionado(pos);
                    int fila= (int)(pos.x()/tanchuraZ);
                    int columna= (int)(pos.y()/talturaZ);
                    if(!posLibre(fila,columna)) /*Si existe un tile en la posición del mapa que refleja el cursor*/
                    {
                        tilescopiar.append(matrizTiles[fila][columna]);
                        poscopiar.append(QPoint(x,y));
                    }
                    int nuevaposal=pos.y()+talturaZ;
                    pos.setY(nuevaposal);
                    y++;
                }
                y=0;
                int nuevaposan=pos.x()+tanchuraZ;
                pos.setX(nuevaposan);
                x++;
                pos.setY(rectsel.top());
            }
        }
        //si solo es un tile el que debemos copiar
        else
        {
            int fila= (int)(rectsel.x()/tanchuraZ);
            int columna= (int)(rectsel.y()/talturaZ);
            if(!posLibre(fila,columna))
            {
                tilescopiar.append(matrizTiles[fila][columna]);
                poscopiar.append(QPoint(x,y));
            }
        }
    }
}

void Mapa::Cortar()
{
    Copiar();
    if(rectsel.isValid())
        Eliminar();
}

void Mapa::Pegar()
{
    //Pegar Tiles
    for(int i=0;i!=tilescopiar.size();i++)
    {
        QPixmap pixmap=listatiles->ImagenTile(tilescopiar.at(i));
        QPoint pos=rectcopiar.topLeft();
        pos.setX(pos.x()+poscopiar.at(i).x()*tanchuraZ);
        pos.setY(pos.y()+poscopiar.at(i).y()*talturaZ);
        QRect rect=TileSeleccionado(pos);

        //si el tile a pegar no excede las dimensiones del mapa
        if(pos.x()<tanchuraZ*tnumanchura && pos.y() < talturaZ*tnumaltura)
        {
            QPoint punto(rect.x()/tanchuraZ,rect.y()/talturaZ);
            matrizTiles[punto.x()][punto.y()]=tilescopiar.at(i);

            minimapa->Actualizar(QPoint(punto.x(),punto.y()),pixmap,true);

            InsertarDeshacer(tilescopiar.at(i),punto);
        }
    }
    InsertarNumDeshacer(0,tilescopiar.size());
    PegarActivar(false);
    update(rectcopiar);
}

void Mapa::Eliminar()
{
    if(rectsel.isValid())
    {
        int cont=0;
        //si la region a eliminar es mayor que el tamaño de un tile, debemos ir eliminando tile a tile perteneciente a esa region
        if(rectsel.width() > tanchuraZ || rectsel.height() > talturaZ)
        {
            QPoint pos = rectsel.topLeft();
            while(pos.x()<=rectsel.right())
            {
                while(pos.y()<=rectsel.bottom())
                {
                    int fila= (int)(pos.x()/tanchuraZ);
                    int columna= (int)(pos.y()/talturaZ);
                    if(!posLibre(fila,columna))
                    {
                        InsertarDeshacer(matrizTiles[fila][columna],QPoint(fila,columna));
                        cont++;
                        matrizTiles[fila][columna]=-1;

                        minimapa->Actualizar(QPoint(fila,columna),false);
                    }
                    int nuevaposal=pos.y()+talturaZ;
                    pos.setY(nuevaposal);
                }
                int nuevaposan=pos.x()+tanchuraZ;
                pos.setX(nuevaposan);
                pos.setY(rectsel.top());
            }
            InsertarNumDeshacer(1,cont);
        }
        else //si solo eliminamos un tile
        {
            int fila= (int)(rectsel.x()/tanchuraZ);
            int columna= (int)(rectsel.y()/talturaZ);
            if(!posLibre(fila,columna))
            {
                    InsertarDeshacer(matrizTiles[fila][columna],QPoint(fila,columna));
                    matrizTiles[fila][columna]=-1;
                    InsertarNumDeshacer(1,1);
             }
        }
        update(rectsel);
    }
}

void Mapa::Redimensionar(int tanchura, int taltura, int tnumanchura, int tnumaltura)
{
    int numanchaux = NumAnchuraTile();
    int numaltaux = NumAlturaTile();
    int limitanc,limitalt;
    //establecemos las nuevas dimesiones
    setAnchuraTile(tanchura);
    setAlturaTile(taltura);
    setNumAnchuraTile(tnumanchura);
    setNumAlturaTile(tnumaltura);
    tanchuraZ=(int)(tanchura*zoomposibles[factorzoom]);
    talturaZ=(int)(taltura*zoomposibles[factorzoom]);
    setFixedSize(tanchuraZ*tnumanchura,talturaZ*tnumaltura);

    int **mtilesaux;

    mtilesaux = new int*[tnumanchura];
    for(int i=0;i<tnumanchura;i++)
    {
        mtilesaux[i] = new int[tnumaltura];
    }

    //los tiles que excedan la nueva dimension deber ser eliminados, para ello establecemos los límites a lo ancho y alto.
    if(numanchaux > tnumanchura) limitanc=tnumanchura; else limitanc=numanchaux;
    if(numaltaux > tnumaltura) limitalt=tnumaltura; else limitalt=numaltaux;

    for(int i=0;i<limitanc;i++)
        for(int j=0;j<limitalt;j++)
        {
                mtilesaux[i][j] = matrizTiles[i][j];
        }
    for(int i=limitanc;i<tnumanchura;i++)
        for(int j=limitalt;j<tnumaltura;j++)
        {
            mtilesaux[i][j] = -1;
        }
    int **ppaux; ppaux=matrizTiles;

    matrizTiles = mtilesaux;
    delete ppaux;

    pintarcompleto=1;
    update();
    minimapa->Redimensionar();
}

 void Mapa::dragEnterEvent(QDragEnterEvent *event)
 {
     if (event->mimeData()->hasFormat("image/piezas-tile"))
     {
         event->accept();
     }
     else
         event->ignore();
 }

 void Mapa::dragLeaveEvent(QDragLeaveEvent *event)
 {
     event->accept();
 }

 void Mapa::dragMoveEvent(QDragMoveEvent *event)
 {
     int fila= (int)(event->pos().x()/tanchuraZ);
     int columna= (int)(event->pos().y()/talturaZ);
     bool vacio = posLibre(fila,columna);
     if (event->mimeData()->hasFormat("image/piezas-tile") && vacio) { //si no existe ningun tile en la posición en la que estamos
         event->setDropAction(Qt::MoveAction);
         event->accept();
     } else {
         event->ignore();
     }
     update(TileSeleccionado(event->pos()));
 }

//********************************************SOLTAR BOTON***************************************************
 void Mapa::dropEvent(QDropEvent *event)
 {
     int fila= (int)(event->pos().x()/tanchuraZ);
     int columna= (int)(event->pos().y()/talturaZ);
     bool vacio = posLibre(fila,columna);

     if (event->mimeData()->hasFormat("image/piezas-tile") && vacio) {

         QByteArray pieceData = event->mimeData()->data("image/piezas-tile");
         QDataStream stream(&pieceData, QIODevice::ReadOnly);
         QRect square = TileSeleccionado(event->pos());
         QPixmap pixmap; stream >> pixmap;
         int numtile; stream >> numtile;
         matrizTiles[fila][columna] = numtile;

         minimapa->Actualizar(QPoint(fila,columna),pixmap,true);

         posmovact.append(QPoint(fila,columna));
         InsertarNumDeshacer(2,1);

         update(square);

         event->setDropAction(Qt::MoveAction);
         event->accept();

     } else {
         event->ignore();
     }
     modificado=1;
 }

 void Mapa::leaveEvent(QEvent *event)
 {
     if(!seleccion)
     {
         update(rectsel);
     }
 }

 void Mapa::mousePressEvent(QMouseEvent *event)
 {
     QRect square = TileSeleccionado(event->pos());

     int fila= (int)(event->pos().x()/tanchuraZ);
     int columna= (int)(event->pos().y()/talturaZ);
     bool vacio = posLibre(fila,columna);

     QString cad = QString("(") + QString::number(columna) + QString(",") + QString::number(fila) + QString(")");
     ventana->setMensageEstado(cad);

     if(event->button()==Qt::RightButton)
     {
           if(pegado)
           {
                PegarActivar(false);
           }
     }
     else{
         if(pegado) //si tenemos accionada la accion pegar
         {
             Pegar();
         }//si no esta pegado activo
         else if(borrar==1)
         {
             Borrar(square);
         }
         else if(relleno==1)
         {
                Rellenar(square);
         }
         else if(rellenoConjunto==true)
         {
             RellenarConjunto(QPoint(fila,columna));
         }
         else if(seleccion)
         {
             if(rectsel != square)
                rectselant=rectsel;
             rectsel = square; rectselsi=square;
             mov=true;
             update(rectselant);
         }
         else //Si no estamos en ninguna accion especial
         {
            /*Si no existe un tile en la posición del mapa que refleja el cursor*/
             if(vacio)
             {
                 int i = listatiles->vistalistatiles->TileSeleccionado();
                 /*Si tenemos un tile seleccionado lo copiamos*/
                 if( i != -1)
                 {
                     QPixmap pixmap = listatiles->ImagenTile(i);

                     //matrizRect[fila][columna] = square;
                     matrizTiles[fila][columna] = i;

                     /*Tile en lista Deshacer*/
                     InsertarDeshacer(i,QPoint(fila,columna));
                     InsertarNumDeshacer(0,1);

                     minimapa->Actualizar(QPoint(fila,columna),pixmap,true);
                     update(TileSeleccionado(event->pos()));
                }
             }
             else //Si existe un tile, lo cogemos para moverlo y soltarlo en la posicion que queramos.
             {
                 int numtile = matrizTiles[fila][columna];
                 QPixmap pixmap= listatiles->ImagenTile(numtile);
                 QPoint posant = QPoint(fila,columna);//Guardamos la posicion para accion deshacer.

                 //Borramos el tile que vamos a mover
                 matrizTiles[fila][columna] = -1;

                 minimapa->Actualizar(QPoint(fila,columna),false);

                 tilemov.append(numtile);
                 posmovant.append(posant);
                 update(square);

                 //Guardamos la imagen en un drag que nos permite mostranos la imagen al lado del cursor
                 QByteArray itemData;
                 QDataStream dataStream(&itemData, QIODevice::WriteOnly);

                 dataStream << pixmap << numtile;// << location;

                 QMimeData *mimeData = new QMimeData; //Establecemos el tipo de datos que se debe corresponder al modelo de la lista de tiles
                 mimeData->setData("image/piezas-tile", itemData);

                 QDrag *drag = new QDrag(this); //Imagen que se mostrara al lado del puntero de ratón
                 drag->setMimeData(mimeData);
                 drag->setHotSpot(event->pos() - square.topLeft());
                 drag->setPixmap(pixmap);

                 if (drag->start(Qt::MoveAction) == 0)
                 {
                     posmovant.pop_back();
                     tilemov.pop_back();

                     matrizTiles[fila][columna] = numtile;

                     minimapa->Actualizar(QPoint(fila,columna),pixmap,true);

                     update(square);
                 }
            }
         }
     }

 }

 void Mapa::mouseMoveEvent(QMouseEvent *event)
 {
     if(posCorrecta(event->pos()))
     {
         int fila= (int)(event->pos().x()/tanchuraZ);
         int columna= (int)(event->pos().y()/talturaZ);

         QString cad = QString("(") + QString::number(columna) + QString(",") + QString::number(fila) + QString(")");
         ventana->setMensageEstado(cad);

         QRect rect = TileSeleccionado(event->pos());
         if(pegado) //Movemos el rectangulo de copiado y actualizamos la antigua y la nueva posición
         {
            QRect rectanterior = rectcopiar;
            rectcopiar=QRect();
            update(rectanterior);
            rectcopiar=rectanterior;
            rectcopiar.moveTo(TileSeleccionado(QPoint(event->pos())).topLeft());
            update(rectcopiar);
         }
         else if(relleno)
         {
             Rellenar(rect);
         }
         else if(borrar)
         {
             Borrar(rect);
         }
         else if(seleccion)
         {
             rectselant=rectsel;
             rectsel = rectselsi.unite(rect); //Unimos la esquina con la el ultimo seleccionado para forma la region
             mov=true;
             update(rectselant);
         }else if(rellenoConjunto){}
         else
         {
             //si no tenemos ninguna accion especial activada sombreamos el tile que el cursor tiene seleccionado y quitamos el sombreo del anterior
             rectselant = rectsel;
             rectsel = rect;
             mov=true;
             update(rectselant);
         }
     }
 }

 void Mapa::paintEvent(QPaintEvent *event)
 {
     int actminimapa=0;
     QPainter painter;
     painter.begin(this);
     painter.setPen(Qt::NoPen);
     //painter.setRenderHint(QPainter::Antialiasing,true);

     if(rejillaactiva)
     {
         QPen pen(crejilla,1,Qt::SolidLine);
         painter.setPen(pen);
         painter.setBrush(Qt::NoBrush);
         painter.fillRect(event->rect(), cfondo);
         painter.drawRect(event->rect());
         if(!rejillapintada) //Pintamos la rejilla en caso de solicitarlo
         {
             //Establecen el estilo con un borde blanco(rejilla).
             for (int i = 0; i <  tnumanchura; ++i)
             {
                for ( int j =0; j < tnumaltura; j++)
                {
                    if(posLibre(i,j))
                    {
                         painter.drawRect(QRect(i*tanchuraZ,j*talturaZ,tanchuraZ,talturaZ));
                    }
                    else
                    {
                         QPixmap pixmap = listatiles->ImagenTile(matrizTiles[i][j]);
                         painter.drawPixmap(RectanguloPosicion(i,j),pixmap);
                    }
                 }
             }
             rejillapintada=true;
         }
     }
     else
     {
            painter.fillRect(event->rect(), cfondo);
     }
     //si la region a actualizar es mayor que un tile, actualizamos tile a tile de la region
     if(event->rect().width() > tanchuraZ || event->rect().height() > talturaZ)
     {
              QPoint pos = event->rect().topLeft();
              while(pos.x()<=event->rect().right())
              {
                  while(pos.y()<=event->rect().bottom())
                  {
                      int fila= (int)(pos.x()/tanchuraZ);
                      int columna= (int)(pos.y()/talturaZ);
                      if(!posLibre(fila,columna))
                      {
                          QPixmap pixmap = listatiles->ImagenTile(matrizTiles[fila][columna]);
                          painter.drawPixmap(RectanguloPosicion(fila,columna),pixmap);
                          actminimapa=1;
                      }
                      if(rejillaactiva)
                      {
                          painter.drawRect(RectanguloPosicion(fila,columna));
                      }
                      pos.setY(pos.y()+talturaZ);
                  }
                  pos.setY(event->rect().top());
                  pos.setX(pos.x()+tanchuraZ);
              }
              if(seleccion){
                      painter.setBrush(csel);
                      painter.drawRect(rectsel);
              }
     }
     else //se trata de 1 solo tile el que debemos pintar
     {
              int fila= (int)(event->rect().x()/tanchuraZ);
              int columna= (int)(event->rect().y()/talturaZ);

              if(!posLibre(fila,columna))
              {
                  QPixmap pixmap = listatiles->ImagenTile(matrizTiles[fila][columna]);
                  if(rejillaactiva)
                  {
                     painter.drawPixmap(RectanguloPosicion(fila,columna),pixmap);
                     painter.drawRect(RectanguloPosicion(fila,columna));
                  }
                  else
                      painter.drawPixmap(RectanguloPosicion(fila,columna),pixmap);
              }
              actminimapa=1;
     }

     if(mov) //si se trata de un movimiento sobre el mapa
     {
         //el tile de la posicion anterior ya a sido pintado de negro antes o con el tile correspondiente en el caso de que hubiera sido pintado ya
         //por tanto pintamos el actual
         if(seleccion){
                 painter.setBrush(csel);
                 painter.drawRect(rectsel);
         }
         else{ //si no estamos en el modo seleccion, pintamos el tile en la posicion si ya a sido pintado anteriormente y luego el cuadro resaltado de movimiento
             int fila= (int)(rectsel.x()/tanchuraZ);
             int columna= (int)(rectsel.y()/talturaZ);
             if(!posLibre(fila,columna)) //si hay un tiles ya pintado hay que pintarlo
             {
                 QPixmap pixmap = listatiles->ImagenTile(matrizTiles[fila][columna]);
                 painter.setBrush(csel);

                 painter.drawPixmap(RectanguloPosicion(fila,columna),pixmap);
                 painter.drawRect(RectanguloPosicion(fila,columna));
             }
             else
             {
                 painter.setBrush(csel);
                 if(rejillaactiva)
                 {
                     painter.drawRect(rectsel);
                 }
                 else
                 {
                     painter.drawRect(rectsel);
                 }
             }
         }
         mov=false;
     }
     else if(pintarcompleto==1) //Si aplicamos zoom debemos pintar todos los tile de nuevo con el nuevo tamaño.
     {
         for (int i = 0; i <  tnumanchura; i++)
         {
            for ( int j =0; j < tnumaltura; j++)
            {
                if(!posLibre(i,j))
                {
                    QPixmap pixmap = listatiles->ImagenTile(matrizTiles[i][j]);
                    painter.drawPixmap(RectanguloPosicion(i,j),pixmap);
                }
                if(rejillaactiva)
                {
                    painter.drawRect(RectanguloPosicion(i,j));
                }
            }
         }
         pintarcompleto=0;
         if(seleccion){
                 painter.setBrush(csel);
                 painter.drawRect(rectsel);
         }
     }
     if(pegado && rectcopiar.isValid())
     {
         painter.setBrush(csel);
         painter.setPen(Qt::NoPen);
         painter.drawRect(rectcopiar.adjusted(1,1,0,0));
     }
     painter.end();
 }

 void Mapa::Zoom(double factor)
 {
     if (factor == 1)
     {
         if(factorzoom != 6)
             factorzoom++;
     }
     else
     {
         if(factorzoom !=0)
            factorzoom--;
     }
     tanchuraZ=(int)(tanchura*zoomposibles[factorzoom]);
     talturaZ=(int)(taltura*zoomposibles[factorzoom]);
     setFixedSize(tanchuraZ*tnumanchura,talturaZ*tnumaltura);

     pintarcompleto=1;
     rectsel=QRect();
     update();
 }

 const QRect Mapa::TileSeleccionado(const QPoint &posicion)
 {
     int x= (int)(posicion.x()/tanchuraZ);
     int y= (int)(posicion.y()/talturaZ);
     return QRect( x*tanchuraZ, y*talturaZ, tanchuraZ, talturaZ);
 }

 Mapa::~Mapa()
 {
     delete matrizTiles;
 }
