/*###############################################################################
## ETiles project
##
## Copyright (C) 2010 Carlos Villegas Núñez  carlos.villegasnunez@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################*/

#include "minimapa.h"

#include <QtGui>

Minimapa::Minimapa(Mapa *mapa, MapaScrollArea *s) : mapa(mapa), scroll(s)
{
    //Calculamos la anchura y altura escalada de cada tile del minimapa(Aproximada a 200x200)
    anchM = mapa->AnchuraTile() * 200 / (mapa->AnchuraTile() * mapa->NumAnchuraTile());
    altM = mapa->AlturaTile() * 200 / (mapa->AlturaTile() * mapa->NumAlturaTile());

    insertar = false;
    borrar = false;

    minanc=anchM*mapa->NumAnchuraTile();
    minalt=altM*mapa->NumAlturaTile();

    setFixedSize(minanc,minalt);
    imagenmapa = QPixmap(minanc,minalt);
    //Establecemos un rectangulo de visión sobre el minimapa segun el tamaño del scroll del mapa para hacer el traslado de vision
    if(mapa->height() <= scroll->height() && mapa->width() <= scroll->width()) //Si es tamaño del mapa es menor que la dimension del scroll
    {
        rectangulo = new QRect(QPoint(0,0),QSize(minanc,minalt));
    }
    else //si es mayor determinamos su tamaño
    {
        int anchura = (scroll->width()-22) * minanc / (mapa->AlturaTile() * mapa->NumAlturaTile());
        int altura = (scroll->height()-22) * minalt / (mapa->AnchuraTile() * mapa->NumAnchuraTile());
        rectangulo = new QRect(QPoint(0,0),QSize(anchura,altura));
    }


    Pintar(QRect(0,0,minanc,minalt));
}

void Minimapa::Redimensionar()
{
    //Calculamos la anchura y altura escalada de cada tile del minimapa(Aproximada a 200x200)
    anchM = mapa->AnchuraTile() * 200 / (mapa->AnchuraTile() * mapa->NumAnchuraTile());
    altM = mapa->AlturaTile() * 200 / (mapa->AlturaTile() * mapa->NumAlturaTile());

    insertar = false;
    borrar = false;

    minanc=anchM*mapa->NumAnchuraTile();
    minalt=altM*mapa->NumAlturaTile();

    setFixedSize(minanc,minalt);
    imagenmapa = QPixmap(minanc,minalt);
    //Establecemos un rectangulo de visión sobre el minimapa segun el tamaño del scroll del mapa para hacer el traslado de vision
    if(mapa->height() <= scroll->height() && mapa->width() <= scroll->width()) //Si es tamaño del mapa es menor que la dimension del scroll
    {
        rectangulo = new QRect(QPoint(0,0),QSize(minanc,minalt));
    }
    else //si es mayor determinamos su tamaño
    {
        int anchura = (scroll->width()-22) * minanc / (mapa->AlturaTile() * mapa->NumAlturaTile());
        int altura = (scroll->height()-22) * minalt / (mapa->AnchuraTile() * mapa->NumAnchuraTile());
        rectangulo = new QRect(QPoint(0,0),QSize(anchura,altura));
    }


    Pintar(QRect(0,0,minanc,minalt));
}

void Minimapa::Pintar(QRect rect)
{
    QPainter painter;

    painter.begin(&imagenmapa);
    painter.setPen(Qt::NoPen);
    if(insertar) //Si es una insercion
    {
        painter.fillRect(rect,Qt::black);
        painter.drawPixmap(rect,taux);
        insertar = false;
    }
    else if(borrar) //Si es una eliminacion
    {
        painter.fillRect(rect,Qt::black);
        borrar = false;
    }
    else //Si no es ninguna de la dos entonces debemos pintar el minimapa al completo
    {
        painter.fillRect(rect,Qt::black);
        for(int i=0;i<mapa->NumAnchuraTile(); i++)
        {
            for(int j=0;j<mapa->NumAlturaTile();j++)
            {
                if(!mapa->posLibre(i,j))
                {
                    painter.drawPixmap(QRect(i*anchM,j*altM,anchM,altM),mapa->ImagenPosicion(mapa->TilePosicion(i,j)));
                }
            }
        }
    }
    setPixmap(imagenmapa);
}

void Minimapa::Actualizar(QPoint pos, QPixmap tile, bool i)
{
    /*Actualiza el estado del minimapa*/
    int postop = pos.y()*altM;//pos.y() * mapa->AlturaTile() * minanc / (mapa->AlturaTile() * mapa->NumAlturaTile());
    int posleft = pos.x()*anchM;//pos.x() * mapa->AnchuraTile() * minalt / (mapa->AnchuraTile() * mapa->NumAnchuraTile());

    taux = tile;
    insertar=i;
    borrar=!i;
    Pintar(QRect(posleft,postop,anchM,altM));
}

/*void Minimapa::ActualizarCuadro()//QScrollArea *area)
{
    int tanc;
    int talt;

    if(mapa->height() <= scroll->height() && mapa->width() <= scroll->width())
    {
        tanc=mapa->height();
        talt=mapa->width();
    }
    else
    {
        tanc = scroll->width()-22;
        talt = scroll->height()-22;
    }
    imagenunida = new QPixmap(*imagenmapa);
    rectangulo->setRect(scroll->horizontalScrollBar()->value(),scroll->verticalScrollBar()->value(),
                       tanc,talt);
    //PintarCuadrado();

    clear();
    setPixmap(imagenunida->scaled(QSize(200,200)));
}*/

void Minimapa::mousePressEvent(QMouseEvent *ev)
{
    int x,y,pospuntx,pospunty;

    if(ev->pos().x() > minanc-rectangulo->size().width()/2)
    {
        x=minanc-rectangulo->size().width();
    }
    else if(ev->pos().x() < rectangulo->size().width()/2)
    {
        x=0;
    }
    else
        x=int(ev->pos().x() - rectangulo->size().width()/2);

    if(ev->pos().y() > minalt-rectangulo->size().height()/2)
    {
        y=minalt-rectangulo->size().height();
    }
    else if(ev->pos().y() < rectangulo->size().height()/2)
    {
        y=0;
    }
    else{
        y=int(ev->pos().y() - rectangulo->size().height()/2);
    }
    pospuntx=int(x*mapa->AnchuraTileZoom()*mapa->NumAnchuraTile()/minanc);
    pospunty=int(y*mapa->AlturaTileZoom()*mapa->NumAlturaTile()/minalt);

    scroll->ActualizarVision(pospuntx,pospunty);
}

