/*###############################################################################
## ETiles project
##
## Copyright (C) 2010 Carlos Villegas Núñez  carlos.villegasnunez@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################*/

#ifndef MINIMAPA_H
#define MINIMAPA_H

#include <QWidget>
#include <QRect>
#include <QString>
#include <QPixmap>
#include <QLabel>
#include <QPainter>
#include <QScrollArea>
#include <QScrollBar>
#include <ventanaprincipal.h>

#include "mapa.h"
#include "mapascrollarea.h"

class QMouseEvent;
class Mapa;

class Minimapa: public QLabel
{
public:
    /** Constructor de la clase Minimapa, inicializa la superficie donde se representa el minimapa
     * @param mapa: Puntero al mapa que representa
     * @param s: Puntero al scroll del mapa que representa
     */
    Minimapa(Mapa *mapa, MapaScrollArea *s);
    /** Procedimiento que procesa los datos necesarios para actualizar una zona del minimapa
     * @param pos: Posición donde se realiza la actualización
     * @param tile: Imagen correspondiente al tile que vamos a actualizar
     * @param i: Booleano que indica si te trata de una inserción o eliminación
    */
    void Actualizar(QPoint pos,QPixmap tile=QPixmap(),bool i=false);
    //void ActualizarCuadro();
    /** Procedimiento que pinta la región actualizada sobre el minimapa
     * @param rect: Rectángulo que indica la zona a actualizar.
    */
    void Pintar(QRect rect=QRect(0,0,200,200));
    void Redimensionar();

protected:
    /** Procedimiento que se activa cada vez que se produce un evento presionar del ratón
     * @param ev: Puntero a los datos del evento del ratón
     */
    void mousePressEvent(QMouseEvent *ev);
private:
    QPixmap imagenmapa,taux;
    QRect *rectangulo;

    Mapa *mapa;
    MapaScrollArea *scroll;

    bool insertar,borrar;
    int contador,anchM,altM,minanc,minalt;
};

#endif // MINIMAPA_H
