/*###############################################################################
## ETiles project
##
## Copyright (C) 2010 Carlos Villegas Núñez  carlos.villegasnunez@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################*/

#include <QtGui>

#include "tile.h"
#include <QList>

 Tile::Tile(QObject *parent) : QAbstractListModel(parent)
 {
     tiles = new QList<QPixmap>();
     modificado=1;
 }

 Tile::Tile(QObject *parent, ListaTiles *lista) : QAbstractListModel(parent), vistalistatiles(lista)
 {
     tiles = new QList<QPixmap>();
     modificado=0;
 }

 QVariant Tile::data(const QModelIndex &index, int role) const
 {
     if (!index.isValid())
         return QVariant();
     if (role == Qt::DecorationRole)
         return QIcon(tiles->value(index.row()).scaled(60,60,Qt::KeepAspectRatio, Qt::SmoothTransformation)); //Copiar en la lista de tiles mismo estilo en la imagen.
     else if (role == Qt::UserRole)
         return tiles->value(index.row()); // Copiar el tile al Mapa con el mismo estilo que el de la lista tiles
     //else if (role == Qt::UserRole + 1)
     //    return locations->value(index.row());

     return QVariant();
 }

 void Tile::addPiece(const QPixmap &pixmap, QString nombret, int sep)
 {
     int pos = tiles->size();

     beginInsertRows(QModelIndex(), pos, pos); //Ultima posicion de la lista
     tiles->insert(pos, pixmap);
     nombres.append(nombret.section('/',-1)); //Añadimos solo el nombre y no la ruta al completo
     separaciones.append(sep);
     endInsertRows();

     modificado=1;
 }

 Qt::ItemFlags Tile::flags(const QModelIndex &index) const
 {
     if (index.isValid())
         return (Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled);

     return Qt::ItemIsDropEnabled;
 }

 bool Tile::removeRow(int row, const QModelIndex &parent)
 {
     if (parent.isValid())
         return false;

     if (row >= tiles->size())
         return false;

    if( nombres.count(nombres.at(row)) > 1) //Si es un tileset, borramos todos los tiles que lo componen
    {
        QString nombre=nombres.at(row);
        int j=0;
        for(int i=0;i<nombres.size()+j;i = i + 1)
        {
            if(nombres.at(i-j)==nombre)
            {
                beginRemoveRows(parent,i-j,i-j);
                nombres.removeAt(i-j);
                tiles->removeAt(i-j);
                endRemoveRows();
                j++;
            }
        }
    }
    else
    {
        beginRemoveRows(parent,row,row);
        tiles->removeAt(row);
        nombres.removeAt(row);
        endRemoveRows();
    }
    return true;
 }

 Tile::~Tile()
 {
     tiles->clear();
 }
