/*###############################################################################
## ETiles project
##
## Copyright (C) 2010 Carlos Villegas Núñez  carlos.villegasnunez@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################*/

#ifndef VENTANAPRINCIPAL_H
#define VENTANAPRINCIPAL_H

#include <QtGui/QMainWindow>
#include <QFile>
#include <QPixmap>
#include <QDockWidget>
#include "minimapa.h"
#include "mapa.h"
#include "listatiles.h"
#include "tile.h"
#include "maleatorios.h"

class Minimapa;
class Mapa;
class ListaTiles;
class Tile;
class MapaScrollArea;

QT_BEGIN_NAMESPACE
class QListView;
class QMdiArea;
class QMdiSubWindow;
class QSignalMapper;
class QScrollArea;
class QScrollBar;
QT_END_NAMESPACE

namespace Ui
{
    class VentanaPrincipal;
}

class VentanaPrincipal : public QMainWindow
{
    Q_OBJECT

public:
    /** Contructor de la clase VentanaPrincipal, que se encarga de inicializar la ventana principal
     * @param parent: Referencia al padre que crea la instancia de la clase
     */
    VentanaPrincipal(QWidget *parent = 0);
    /** Destructor de la clase Ventana Principal*/
    ~VentanaPrincipal();
    /** Procedimiento que se encarga de crear la ventana donde se gestionará el mapa normal*/
    void CrearNuevoMapa(double tanchura,double taltura, double tnumanchura, double tnumaltura);
    /** Procedimiento que se encarga de crear la ventana del mapa y dicho mapa de forma aleatoria*/
    void CrearMapaAleatorio(double tanchura,double taltura, double tnumanchura, double tnumaltura,QList<QPixmap> ltiles,QList<QString>, int aguatierra, int contislas, int monvar, int semilla,int eleccion);
    /** Procedimiento que se encarga de crear la lista de tile y su ventana de gestión*/
    void CrearVentanaTiles();
    /** Procedimiento que se encarga de crear el minimapa y su ventana de gestión*/
    void CrearMinimapa();
    /** Procedimiento que llama a dicha función sobre el mapa*/
    void RedimensionarMapa(int tanchura, int taltura, int tnumanchura, int tnumaltura);
    /** Procedimiento que guarda el proyecto en un fichero*/
    void GuardarProyecto(int i);
    /** Procedimiento que carga un proyecto de un fichero*/
    void CargarProyecto();
    /** Procedimiento que exporta el mapa de un proyecto a un fichero con un formato especifico*/
    void Exportar();
    /** Procedimiento que elimina datos dinámicos cuando se crea un nuevo mapa sobre otro*/
    void Limpiar();
    /** Procedimiento que establece un mensaje en la barra de estado*/
    void setMensageEstado(QString msg);
    /** Método constante por el cual se obtiene un puntero al mapa que se representa en la ventana principal*/
    Mapa* getMapa() const { return mapa;}
    /** Método constante por el cual se obtiene un puntero a la lista de tiles que se representa en la ventana principal*/
    Tile* getTile() const { return tile;}

private slots:
    void RellenoActivado();
    void RellenoConjuntoActivado();
    void BorrarActivado();
    void SeleccionActivada();
    void EliminarTileArea();
    void AnadirTileset();
    void on_actionNormal_activated();
    void on_actionAleatorio_activated();
    void on_actionCargar_activated();
    void on_actionGuardar_activated();
    void on_actionGuardar_Como_activated();
    void on_actionExportar_activated();
    void on_actionImportar_activated();
    void on_actionSalir_activated();
    void on_actionCopiar_activated();
    void on_actionCortar_activated();
    void on_actionPegar_activated();
    void on_actionEliminar_activated();
    void on_actionZoomIn_activated();
    void on_actionZoomOut_activated();
    void on_actionMapa_activated();
    void on_actionRejilla_activated();
    void on_actionRehacer_activated();
    void on_actionDeshacer_activated();
    void on_actionRedimensionar_activated();
    void on_actionPropiedades_activated();

private:
    /** Procedimiento que aplica el método Perlin Noise para obtener el mapa aleatorio*/
    void AleatorioPerlin(int octaves, double frecuencia, double amplitud, int semilla,double margen);

    Ui::VentanaPrincipal *ui;
    QAction *nuevo,*guardar,*cargar,*deshacer,*rehacer,*seleccion,*borrar,*relleno,*rellenoConjunto,*zoommas,*zoommenos;

    QDockWidget *areatiles,*areaminimapa;

    Minimapa *minimapa;
    Mapa *mapa;
    Tile *tile;
    ListaTiles *listatiles;
    QString *nombremapa;
    int inicio,inicioguardado;

    QScrollArea scrollminimapa;
    MapaScrollArea *scrollAreamapa;

    QLineEdit prueba2;
};

#endif // VENTANAPRINCIPAL_H
